use std::time::Duration;

use assert2::let_assert;
use git_next_core::git::UserNotification;

use crate::{alerts::History, repo::tests::given};

#[test]
fn when_history_is_empty_then_message_is_passed() {
    let mut history = History::new(Duration::from_millis(1));

    let user_notification = UserNotification::RepoConfigLoadFailure {
        forge_alias: given::a_forge_alias(),
        repo_alias: given::a_repo_alias(),
        reason: given::a_name(),
    };

    let result = history.sendable(user_notification);

    assert!(result.is_some());
}

#[test]
fn when_history_has_expired_then_message_is_passed() {
    let dur = Duration::from_millis(1);
    let mut history = History::new(dur);

    let user_notification = UserNotification::RepoConfigLoadFailure {
        forge_alias: given::a_forge_alias(),
        repo_alias: given::a_repo_alias(),
        reason: given::a_name(),
    };

    // add message to history
    let result = history.sendable(user_notification);
    let_assert!(Some(user_notification) = result);

    std::thread::sleep(dur);
    // twice the expiry duration to avoid things happening in the wrong order
    std::thread::sleep(dur);

    // after dur, message has expired, so is still valid
    let result = history.sendable(user_notification);
    assert!(result.is_some());
}

#[test]
fn when_history_has_unexpired_then_message_is_blocked() {
    let dur = Duration::from_secs(1);
    let mut history = History::new(dur);

    let user_notification = UserNotification::RepoConfigLoadFailure {
        forge_alias: given::a_forge_alias(),
        repo_alias: given::a_repo_alias(),
        reason: given::a_name(),
    };

    // add message to history
    let result = history.sendable(user_notification);
    let_assert!(Some(user_notification) = result);

    // no time passed
    // std::thread::sleep(dur);

    // after dur, message has expired, so is still valid
    let result = history.sendable(user_notification);
    assert!(result.is_none());
}
