//
#[cfg(any(feature = "forgejo", feature = "github"))]
use super::*;

use git_next_core::{
    self as core,
    git::{self, RepoDetails},
    GitDir, RepoConfigSource, StoragePathType,
};

#[cfg(feature = "forgejo")]
#[test]
fn test_forgejo_name() {
    let mock_net = kxio::net::mock();
    let repo_details = given_repo_details(git_next_core::ForgeType::ForgeJo);
    let forge = Forge::create(repo_details, mock_net.clone().into());
    assert_eq!(forge.name(), "forgejo");
    mock_net.assert_no_unused_plans();
}

#[cfg(feature = "github")]
#[test]
fn test_github_name() {
    let mock_net = kxio::net::mock();
    let repo_details = given_repo_details(git_next_core::ForgeType::GitHub);
    let forge = Forge::create(repo_details, mock_net.clone().into());
    assert_eq!(forge.name(), "github");
    mock_net.assert_no_unused_plans();
}

#[allow(dead_code)]
fn given_repo_details(forge_type: git_next_core::ForgeType) -> RepoDetails {
    let fs = kxio::fs::temp().unwrap_or_else(|e| {
        println!("{e}");
        panic!("fs")
    });
    git::repo_details(
        1,
        git::Generation::default(),
        core::common::forge_details(1, forge_type),
        Some(core::common::repo_config(1, RepoConfigSource::Repo)),
        GitDir::new(fs.base().to_path_buf(), StoragePathType::Internal),
    )
}
