use git_next_core::git::push::Force;
use git_next_core::git::GitRef;

//
use super::*;

mod advance_main;
mod advance_next;

use crate::git;
use crate::repo::branch;

#[tokio::test]
async fn test_find_next_commit_on_dev_when_next_is_at_main() {
    let next = given::a_commit(); // and main
    let expected = given::a_commit();
    let dev_commit_history = vec![
        given::a_commit(), // dev HEAD
        expected.clone(),
        next.clone(),      // next - advancing towards dev HEAD
        given::a_commit(), // parent of next
    ];

    let (next_commit, force) = branch::find_next_commit_on_dev(&next, &next, &dev_commit_history);

    assert_eq!(next_commit, Some(expected), "Found the wrong commit");
    assert_eq!(force, Force::No, "should not try to force");
}

#[tokio::test]
async fn test_find_next_commit_on_dev_when_next_is_not_on_dev() {
    let next = given::a_commit();
    let main = given::a_commit();
    let expected = given::a_commit();
    let dev_commit_history = vec![
        given::a_commit(), // dev HEAD
        expected.clone(),
        main.clone(),      // main - advancing towards dev HEAD
        given::a_commit(), // parent of next
    ];

    let (next_commit, force) = branch::find_next_commit_on_dev(&next, &main, &dev_commit_history);

    assert_eq!(next_commit, Some(expected), "Found the wrong commit");
    assert_eq!(
        force,
        Force::From(GitRef::from(next.sha().clone())),
        "should force back onto dev branch"
    );
}
