use crate::{repo::messages::CheckCIStatus, tell};

//
use super::*;

#[tokio::test]
async fn should_passthrough_to_receive_ci_status() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);
    let next_commit = given::a_named_commit("next");
    let mut forge = git::MockForgeLike::new();
    forge
        .expect_commit_status()
        .with(mockall::predicate::eq(next_commit.clone()))
        .return_once(|_| Ok(git::forge::commit::Status::Pass));

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        Box::new(forge),
    );
    tell!(addr, CheckCIStatus::new(next_commit.clone()))?;

    //then
    tracing::debug!(?log, "");
    log.require_message_containing("send: ReceiveCIStatus")
        .await?;
    Ok(())
}
