//
use crate::{repo::messages::ReceiveRepoConfig, tell};

use super::*;

#[tokio::test]
async fn should_store_repo_config_in_actor() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);

    let new_repo_config = given::a_repo_config();

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        given::a_forge(),
    );
    tell!(addr, ReceiveRepoConfig::new(new_repo_config.clone()))?;

    //then
    tracing::debug!(?log, "");

    let reo_actor_view = addr
        .ask(ExamineActor)
        .await
        .map_err(|e| format!("examine actor: {e:?}"))?;
    assert_eq!(
        reo_actor_view.repo_details.repo_config,
        Some(new_repo_config)
    );
    Ok(())
}

#[test_log::test(tokio::test)]
async fn should_register_webhook() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);

    let new_repo_config = given::a_repo_config();

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        given::a_forge(),
    );
    tell!(addr, ReceiveRepoConfig::new(new_repo_config.clone()))?;

    //then
    tracing::debug!(?log, "");
    log.require_message_containing("send: RegisterWebhook")
        .await?;
    Ok(())
}
