use crate::{repo::messages::RegisterWebhook, tell};

//
use super::*;

#[tokio::test]
async fn when_registered_ok_should_send_webhook_registered() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);

    let registered_webhook = given::a_registered_webhook();
    let mut forge = git::MockForgeLike::new();
    forge
        .expect_register_webhook()
        .return_once(move |_| Ok(registered_webhook));

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        Box::new(forge),
    );
    tell!(addr, RegisterWebhook::new())?;
    //then
    tracing::debug!(?log, "");
    log.require_message_containing("send: WebhookRegistered")
        .await?;
    Ok(())
}

#[tokio::test]
async fn when_registered_error_should_send_notify_user() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);

    let mut forge = git::MockForgeLike::new();
    forge.expect_register_webhook().return_once(move |_| {
        Err(git::forge::webhook::Error::FailedToRegister(
            "foo".to_string(),
        ))
    });

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        Box::new(forge),
    );
    tell!(addr, crate::repo::messages::RegisterWebhook::new())?;

    //then
    tracing::debug!(?log, "");
    log.require_message_containing("send: NotifyUser").await?;
    Ok(())
}
