use crate::{repo::messages::WebhookRegistered, tell};

//
use super::*;

#[tokio::test]
async fn should_store_webhook_details() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);
    let webhook_id = given::a_webhook_id();
    let webhook_auth = given::a_webhook_auth();

    //when
    let (addr, _log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        given::a_forge(),
    );
    tell!(
        addr,
        WebhookRegistered::new((webhook_id.clone(), webhook_auth.clone()))
    )?;

    //then
    // let view = addr.ask(ExamineActor).await.expect("examine actor");
    let view = addr
        .ask(ExamineActor)
        .await
        .map_err(|e| format!("examine actor: {e:?}"))?;
    assert_eq!(view.webhook_id, Some(webhook_id));
    assert_eq!(view.webhook_auth, Some(webhook_auth));
    Ok(())
}

#[tokio::test]
async fn should_send_validate_repo_message() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);
    let webhook_id = given::a_webhook_id();
    let webhook_auth = given::a_webhook_auth();

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        given::a_forge(),
    );
    tell!(
        addr,
        WebhookRegistered::new((webhook_id.clone(), webhook_auth.clone()))
    )?;

    //then
    log.require_message_containing("send: ValidateRepo").await?;
    Ok(())
}
