type TestResult = Result<(), Box<dyn std::error::Error>>;

mod init {
    use super::*;
    #[test]
    fn should_not_update_file_if_it_exists() -> TestResult {
        let fs = kxio::fs::temp()?;
        let file = fs.base().join(".git-next.toml");
        fs.file(&file).write("contents")?;

        crate::init::run(&fs)?;

        assert_eq!(
            fs.file(&file).reader()?.to_string(),
            "contents",
            "The file has been changed"
        );

        Ok(())
    }

    #[test]
    fn should_create_default_file_if_not_exists() -> TestResult {
        let fs = kxio::fs::temp()?;

        crate::init::run(&fs)?;

        let file = fs.base().join(".git-next.toml");

        assert!(fs.path(&file).exists()?, "The file has not been created");

        assert_eq!(
            fs.file(&file).reader()?.to_string(),
            include_str!("../default.toml"),
            "The file does not match the default template"
        );

        Ok(())
    }
}
