//
mod model {
    mod repo_state {
        use git_next_core::{git::graph::Log, RepoBranches};
        use ratatui::style::Style;

        use crate::{
            repo::tests::given,
            tui::actor::{RepoMessage, RepoState, ViewState},
        };
        type Alert = Option<String>;

        fn identified_with_alert(alert: Alert) -> RepoState {
            RepoState::Identified {
                repo_alias: given::a_repo_alias(),
                message: RepoMessage::builder()
                    .text(given::a_name())
                    .style(Style::default())
                    .build(),
                alert,
            }
        }

        fn configured_with_alert(alert: Alert) -> RepoState {
            RepoState::Configured {
                repo_alias: given::a_repo_alias(),
                message: RepoMessage::builder()
                    .text(given::a_name())
                    .style(Style::default())
                    .build(),
                alert,
                branches: RepoBranches::new(String::new(), String::new(), String::new()),
                log: Log::default(),
            }
        }
        fn ready_with_alert(alert: Alert) -> RepoState {
            RepoState::Ready {
                repo_alias: given::a_repo_alias(),
                message: RepoMessage::builder()
                    .text(given::a_name())
                    .style(Style::default())
                    .build(),
                alert,
                branches: RepoBranches::new(String::new(), String::new(), String::new()),
                log: Log::default(),
                view_state: ViewState::default(),
                main: given::a_commit(),
                next: given::a_commit(),
                dev: given::a_commit(),
            }
        }

        #[rstest::rstest]
        #[case(identified_with_alert(None))]
        #[case(configured_with_alert(None))]
        #[case(ready_with_alert(None))]
        fn none_alert_remains_none(#[case] mut repo_state: RepoState) {
            // given
            match &repo_state {
                RepoState::Identified { alert, .. }
                | RepoState::Configured { alert, .. }
                | RepoState::Ready { alert, .. } => {
                    assert!(alert.is_none(), "should be none at start");
                }
            }
            // when
            repo_state.clear_alert();
            // then
            match &repo_state {
                RepoState::Identified { alert, .. }
                | RepoState::Configured { alert, .. }
                | RepoState::Ready { alert, .. } => assert!(alert.is_none(), "should remain none"),
            }
        }

        #[rstest::rstest]
        #[case(identified_with_alert(Some(String::new())))]
        #[case(configured_with_alert(Some(String::new())))]
        #[case(ready_with_alert(Some(String::new())))]
        fn some_alert_becomes_none(#[case] mut repo_state: RepoState) {
            // given
            match &repo_state {
                RepoState::Identified { alert, .. }
                | RepoState::Configured { alert, .. }
                | RepoState::Ready { alert, .. } => {
                    assert!(alert.is_some(), "should be some at start");
                }
            }
            // when
            repo_state.clear_alert();
            // then
            match &repo_state {
                RepoState::Identified { alert, .. }
                | RepoState::Configured { alert, .. }
                | RepoState::Ready { alert, .. } => assert!(alert.is_none(), "should become none"),
            }
        }
    }
}
