use super::*;

#[test]
fn should_return_repos() {
    let forge_type = ForgeType::MockForge;
    let hostname = given::a_name();
    let user = given::a_name();
    let token = given::a_name();
    // alphabetical order by key
    let red_name = format!("a-{}", given::a_name());
    let blue_name = format!("b-{}", given::a_name());
    let red = ServerRepoConfig::new(red_name.clone(), given::a_name(), None, None, None, None);
    let blue = ServerRepoConfig::new(blue_name.clone(), given::a_name(), None, None, None, None);
    let mut repos = BTreeMap::new();
    repos.insert(red_name.clone(), red.clone());
    repos.insert(blue_name.clone(), blue.clone());
    let fc = ForgeConfig::new(
        forge_type,
        hostname,
        user,
        token,
        given::maybe_a_number(),
        repos,
    );

    let returned_repos = fc.repos().collect::<Vec<_>>();

    assert_eq!(
        returned_repos,
        vec![
            // alphabetical order by key
            (RepoAlias::new(red_name.as_str()), &red),
            (RepoAlias::new(blue_name.as_str()), &blue),
        ]
    );
}

#[test]
fn should_return_forge_type() {
    let forge_type = ForgeType::MockForge;
    let hostname = given::a_name();
    let user = given::a_name();
    let token = given::a_name();
    let repos = BTreeMap::new();
    let fc = ForgeConfig::new(
        forge_type,
        hostname,
        user,
        token,
        given::maybe_a_number(),
        repos,
    );

    assert_eq!(fc.forge_type(), ForgeType::MockForge);
}

#[test]
fn should_return_hostname() {
    let forge_type = ForgeType::MockForge;
    let hostname = given::a_name();
    let user = given::a_name();
    let token = given::a_name();
    let repos = BTreeMap::new();
    let fc = ForgeConfig::new(
        forge_type,
        hostname.clone(),
        user,
        token,
        given::maybe_a_number(),
        repos,
    );

    assert_eq!(fc.hostname(), Hostname::new(hostname));
}

#[test]
fn should_return_user() {
    let forge_type = ForgeType::MockForge;
    let hostname = given::a_name();
    let user = given::a_name();
    let token = given::a_name();
    let repos = BTreeMap::new();
    let fc = ForgeConfig::new(
        forge_type,
        hostname,
        user.clone(),
        token,
        given::maybe_a_number(),
        repos,
    );

    assert_eq!(fc.user(), User::new(user));
}

#[test]
fn should_return_token() {
    let forge_type = ForgeType::MockForge;
    let hostname = given::a_name();
    let user = given::a_name();
    let token = given::a_name();
    let repos = BTreeMap::new();
    let fc = ForgeConfig::new(
        forge_type,
        hostname,
        user,
        token.clone(),
        given::maybe_a_number(),
        repos,
    );

    assert_eq!(fc.token().expose_secret(), token.as_str());
}

#[test]
fn should_return_repo() {
    let forge_type = ForgeType::MockForge;
    let hostname = given::a_name();
    let user = given::a_name();
    let token = given::a_name();
    let red_name = given::a_name();
    let blue_name = given::a_name();
    let red = ServerRepoConfig::new(red_name.clone(), given::a_name(), None, None, None, None);
    let blue = ServerRepoConfig::new(blue_name.clone(), given::a_name(), None, None, None, None);

    let mut repos = BTreeMap::new();
    repos.insert(red_name.clone(), red.clone());
    repos.insert(blue_name, blue);
    let fc = ForgeConfig::new(
        forge_type,
        hostname,
        user,
        token,
        given::maybe_a_number(),
        repos,
    );

    let returned_repo = fc.get_repo(red_name.as_str());

    assert_eq!(returned_repo, Some(&red),);
}
