use std::ops::Deref as _;

//
use super::*;

#[test]
// assumes running in the git-next repo which should have main, next and dev as remote branches
fn should_return_file() -> TestResult {
    let_assert!(Ok(fs) = kxio::fs::temp());
    let repo_config = given::a_repo_config();
    let file_name = given::a_pathbuf();
    let contents = given::a_name();
    let gitdir = GitDir::new(fs.base().to_path_buf(), StoragePathType::Internal);
    let forge_details = given::forge_details();

    let test_repository = git::repository::test(fs.deref().clone(), forge_details);
    let_assert!(Ok(open_repository) = test_repository.open(&gitdir));
    then::commit_named_file_to_branch(
        &file_name,
        &contents,
        &fs,
        &gitdir,
        &repo_config.branches().main(),
    )?;
    // then::create_a_commit_on_branch(&fs, &gitdir, &repo_config.branches().main())?;
    let_assert!(
        Ok(result) = open_repository.read_file(&repo_config.branches().main(), &file_name),
        "read file"
    );
    assert_eq!(result, contents);
    Ok(())
}

#[test]
// assumes running in the git-next repo which should have main, next and dev as remote branches
fn should_error_on_missing_file() -> TestResult {
    let_assert!(Ok(fs) = kxio::fs::temp());
    let gitdir = GitDir::new(fs.base().to_path_buf(), StoragePathType::Internal);
    let forge_details = given::forge_details();
    let test_repository = git::repository::test(fs.deref().clone(), forge_details);
    let_assert!(Ok(open_repository) = test_repository.open(&gitdir));
    let repo_config = &given::a_repo_config();
    let branches = repo_config.branches();
    then::create_a_commit_on_branch(&fs, &gitdir, &branches.dev())?;
    let_assert!(
        Err(err) = open_repository.read_file(&branches.dev(), &given::a_pathbuf()),
        "read file"
    );
    eprintln!("err: {err:#?}");
    assert!(matches!(err, git::file::Error::FileNotFound));
    Ok(())
}
