//
use super::*;

#[test]
fn should_not_return_repo_config_when_no_branches() {
    let main = None;
    let next = None;
    let dev = None;
    let src = ServerRepoConfig::new(given::a_name(), given::a_name(), None, main, next, dev);

    let_assert!(None = src.repo_config());
}

#[test]
fn should_return_repo_config_when_branches() {
    let main = given::a_name();
    let next = given::a_name();
    let dev = given::a_name();
    let src = ServerRepoConfig::new(
        given::a_name(),
        given::a_name(),
        None,
        Some(main.clone()),
        Some(next.clone()),
        Some(dev.clone()),
    );

    let_assert!(Some(rc) = src.repo_config());
    assert_eq!(
        rc,
        RepoConfig::new(RepoBranches::new(main, next, dev), RepoConfigSource::Server)
    );
}

#[test]
fn should_return_repo() {
    let repo_path = given::a_name();
    let src = ServerRepoConfig::new(
        repo_path.clone(),
        given::a_name(),
        None,
        Some(given::a_name()),
        Some(given::a_name()),
        Some(given::a_name()),
    );

    assert_eq!(src.repo(), RepoPath::new(repo_path));
}

#[test]
fn should_return_branch() {
    let branch = given::a_name();
    let src = ServerRepoConfig::new(
        given::a_name(),
        branch.clone(),
        None,
        Some(given::a_name()),
        Some(given::a_name()),
        Some(given::a_name()),
    );

    assert_eq!(src.branch(), BranchName::new(branch));
}

#[test]
fn should_return_gitdir() {
    let gitdir = given::a_name();
    let src = ServerRepoConfig::new(
        given::a_name(),
        given::a_name(),
        Some(gitdir.clone().into()),
        Some(given::a_name()),
        Some(given::a_name()),
        Some(given::a_name()),
    );

    assert_eq!(
        src.gitdir(),
        Some(GitDir::new(
            PathBuf::default().join(gitdir),
            StoragePathType::External,
        ))
    );
}
